"use strict";
/**
 * Copyright ou © ou Copr. Ministère de l'Europe et des Affaires étrangères (2017)
 * <p/>
 * pole-architecture.dnum-psi@diplomatie.gouv.fr
 * <p/>
 * Ce logiciel est un programme informatique servant à faciliter la création
 * d'applications Web conformément aux référentiels généraux français : RGI, RGS et RGAA
 * <p/>
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * <p/>
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 * <p/>
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 * <p/>
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 * <p/>
 * <p/>
 * Copyright or © or Copr. Ministry for Europe and Foreign Affairs (2017)
 * <p/>
 * pole-architecture.dnum-psi@diplomatie.gouv.fr
 * <p/>
 * This software is a computer program whose purpose is to facilitate creation of
 * web application in accordance with french general repositories : RGI, RGS and RGAA.
 * <p/>
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 * <p/>
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 * <p/>
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 * <p/>
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Scope = exports.Side = exports.injectable = void 0;
/**
 * hornet-js-core - Ensemble des composants qui forment le coeur de hornet-js
 *
 * @author MEAE - Ministère de l'Europe et des Affaires étrangères
 * @version v5.5.0
 * @link git+https://github.com/diplomatiegouvfr/hornet-js.git
 * @license CECILL-2.1
 */
require("reflect-metadata");
const hornet_js_utils_1 = require("hornet-js-utils");
const injector_1 = require("hornet-js-core/src/inject/injector");
/**
 * Decorateur de classe pour l'enregistrer dans l'injecteur que si les côté correspond et permet d'injecter
 * des valeurs dans le constructeur
 * @param  key {any} clé de stockage
 * @param  side {Side} complément de clé correspondant au côté d'exécution (Client ou Serveur)
 * */
function injectable(key, scope = Scope.VALUE, side) {
    return function (constructor, ...restOfName) {
        if (!restOfName[0] && typeof constructor == "function") { // decorator on class
            let newConstructor = constructor;
            const injectParameters = Reflect.getOwnMetadata(injector_1.INJECT_METADATA_KEY, constructor);
            if (injectParameters) {
                newConstructor = class extends constructor {
                    constructor(...args) {
                        injectParameters.forEach((injectParameter) => {
                            args[injectParameter.index] = injector_1.Injector.getRegistered(injectParameter.type || args[injectParameter.index]);
                        });
                        super(...args);
                    }
                };
            }
            if (key) {
                if (!side || (side === Side.SERVER && hornet_js_utils_1.Utils.isServer) || (side === Side.CLIENT && !hornet_js_utils_1.Utils.isServer)) {
                    injector_1.Injector.register(key, newConstructor, scope);
                }
            }
            return newConstructor;
        }
        else if (typeof constructor[restOfName[0]] == "function") { // decorator on function
            const injectParameters = Reflect.getOwnMetadata(injector_1.INJECT_METADATA_KEY, constructor, restOfName[0]);
            const method = restOfName[1].value;
            restOfName[1].value = function (...args) {
                injectParameters.forEach((injectParameter) => {
                    args[injectParameter.index] = injector_1.Injector.getRegistered(injectParameter.type || args[injectParameter.index]);
                });
                // invoke greet() and get its return value
                return method.apply(this, args);
            };
            return restOfName[1];
        }
        else {
            throw new Error("unknow type for injectable decorator");
        }
    };
}
exports.injectable = injectable;
var Side;
(function (Side) {
    Side[Side["SERVER"] = 1] = "SERVER";
    Side[Side["CLIENT"] = 2] = "CLIENT";
})(Side = exports.Side || (exports.Side = {}));
var Scope;
(function (Scope) {
    Scope[Scope["PROTOTYPE"] = 1] = "PROTOTYPE";
    Scope[Scope["SINGLETON"] = 2] = "SINGLETON";
    Scope[Scope["SINGLETON_EAGER"] = 3] = "SINGLETON_EAGER";
    Scope[Scope["VALUE"] = 4] = "VALUE";
})(Scope = exports.Scope || (exports.Scope = {}));

//# sourceMappingURL=injectable.js.map
