"use strict";
/**
 * Copyright ou © ou Copr. Ministère de l'Europe et des Affaires étrangères (2017)
 * <p/>
 * pole-architecture.dnum-psi@diplomatie.gouv.fr
 * <p/>
 * Ce logiciel est un programme informatique servant à faciliter la création
 * d'applications Web conformément aux référentiels généraux français : RGI, RGS et RGAA
 * <p/>
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * <p/>
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 * <p/>
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 * <p/>
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 * <p/>
 * <p/>
 * Copyright or © or Copr. Ministry for Europe and Foreign Affairs (2017)
 * <p/>
 * pole-architecture.dnum-psi@diplomatie.gouv.fr
 * <p/>
 * This software is a computer program whose purpose is to facilitate creation of
 * web application in accordance with french general repositories : RGI, RGS and RGAA.
 * <p/>
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 * <p/>
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 * <p/>
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 * <p/>
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.HornetRouter = exports.HornetMiddlewareList = exports.DEFAULT_HORNET_API_MIDDLEWARES = exports.DEFAULT_HORNET_MODULE_MIDDLEWARES = exports.DEFAULT_HORNET_MIDDLEWARES = exports.AliveMiddleware = exports.UnmanagedDataErrorMiddleware = exports.DataRenderingSubMiddleware = exports.DataRenderingMiddleware = exports.MenuConfigMiddleware = exports.UserAccessSecurityMiddleware = exports.RouterServerSubMiddleware = exports.RouterServerMiddleware = exports.wakeUpNodeSessionMiddleware = exports.ExpandedLayoutHandlerMiddleware = exports.IsExpandedLayoutMiddleware = exports.SetExpandedLayoutMiddleware = exports.ChangeI18nLocaleMiddleware = exports.InternationalizationMiddleware = exports.CsrfMiddleware = exports.SecurityMiddleware = exports.MulterMiddleware = exports.SessionMiddleware = exports.BodyParserUrlEncodedMiddleware = exports.BodyParserJsonMiddleware = exports.StaticPathErrorMiddleware = exports.StaticPathMiddleware = exports.StaticNodeHttpHeaderMiddleware = exports.WithoutSlashPageRedirectMiddleware = exports.WelcomePageRedirectMiddleware = exports.LoggerUserMiddleware = exports.LoggerTIDMiddleware = exports.HornetContextInitializerSubMiddleware = exports.HornetContextInitializerMiddleware = exports.AbstractHornetSubMiddleware = exports.AbstractHornetMiddleware = void 0;
/**
 * hornet-js-core - Ensemble des composants qui forment le coeur de hornet-js
 *
 * @author MEAE - Ministère de l'Europe et des Affaires étrangères
 * @version v5.5.0
 * @link git+https://github.com/diplomatiegouvfr/hornet-js.git
 * @license CECILL-2.1
 */
const hornet_js_utils_1 = require("hornet-js-utils");
const logger_1 = require("hornet-js-logger/src/logger");
const path = require("path");
const timer_1 = require("hornet-js-core/src/timers/timer");
const isUndefined = require("lodash.isundefined");
const some = require("lodash.some");
const cloneDeep = require("lodash.clonedeep");
const merge = require("lodash.merge");
const isEnvProduction = (process.env["NODE_ENV"] === "production");
function isSecurityEnabled(envProduction, logger) {
    let enabled = false;
    if (hornet_js_utils_1.Utils.config.get("security")) {
        if (hornet_js_utils_1.Utils.config.get("security.enabled") || envProduction) {
            enabled = true;
            logger.trace("SECURITY MIDDLEWARE CONFIGURATION :  Configuration de la sécurité...");
        }
        else {
            logger.warn("SECURITY MIDDLEWARE CONFIGURATION : La sécurité est désactivé!!!");
        }
    }
    else {
        logger.warn("SECURITY MIDDLEWARE CONFIGURATION : La sécurité n'est pas configurée!!!");
    }
    return enabled;
}
function checkSecurityConfiguration(key, enabledKey, middlewareName, logger) {
    let enabled = false;
    if (hornet_js_utils_1.Utils.config.has(key)) {
        if (enabledKey) {
            key = key + ".enabled";
        }
        if (hornet_js_utils_1.Utils.config.getOrDefault(key, true)) {
            logger.debug("SECURITY MIDDLEWARE CONFIGURATION : ajout du middleware " + middlewareName);
            enabled = true;
        }
        else {
            logger.warn("SECURITY MIDDLEWARE CONFIGURATION : Le middleware " + middlewareName + " n'est pas activé");
        }
    }
    else {
        logger.warn("SECURITY MIDDLEWARE CONFIGURATION : Le middleware " + middlewareName + " n'est pas configuré");
    }
    return enabled;
}
function sanitizeErrorThrownInDomain(error) {
    if (error) {
        delete error["error@context"];
        delete error["domain"];
    }
}
// ------------------------------------------------------------------------------------------------------------------- //
/**
 * Classe abstraite de gestion d'un middleware dans une application Hornet
 * Cette classe doit être héritée par chaque middleware
 *
 * Attention : la classe héritant de AbstractHornetMiddleware doit définir le constructeur suivant :
 *          constructor() {
 *              [...]
 *              super(appConfig, PREFIX_OPTIONNEL, FN_OPTIONNEL);
 *              [...]
 *          }
 *
 * > Par défaut, la fonction "insertMiddleware(app)" utilise le PREFIX et FN fournis : app.use(PREFIX, FN)
 *
 * > Pour des cas particuliers, il est possible de surcharger cette méthode pour ajouter autant de middlewares qu'on veut :
 *      public insertMiddleware(app) {
 *          app.use("/exemple", (req, res, next) => { [ ... ] });
 *          app.use((req, res, next) => { .... });
 *          ...
 *      }
 */
class AbstractHornetMiddleware {
    /**
     * Constructeur
     *
     * @param middlewareFunction
     * @param prefix
     * @param config server configuration
     */
    constructor(middlewareFunction, prefix, config) {
        this.prefix = prefix;
        this.middlewareFunction = middlewareFunction;
        this.config = config;
    }
    /**
     * Méthode appelée automatiquement lors de l'initialisation du serveur afin d'ajouter un middleware
     * @param app
     */
    insertMiddleware(app) {
        if (this.prefix !== null && this.prefix !== undefined) {
            app.use(this.prefix, this.middlewareFunction);
        }
        else {
            app.use(this.middlewareFunction);
        }
    }
}
exports.AbstractHornetMiddleware = AbstractHornetMiddleware;
class AbstractHornetSubMiddleware extends AbstractHornetMiddleware {
    insertRouterMiddleware(router) {
        if (this.prefix !== null && this.prefix !== undefined) {
            router.use(this.prefix, this.middlewareFunction);
        }
        else {
            router.use(this.middlewareFunction);
        }
    }
}
exports.AbstractHornetSubMiddleware = AbstractHornetSubMiddleware;
// ------------------------------------------------------------------------------------------------------------------- //
//                                      HornetContextInitializerMiddleware
// ------------------------------------------------------------------------------------------------------------------- //
const abstract_routes_1 = require("hornet-js-core/src/routes/abstract-routes");
class HornetContextInitializerMiddleware extends AbstractHornetMiddleware {
    constructor() {
        const callbacksStorage = hornet_js_utils_1.Utils.getContinuationStorage();
        const appConfig = AbstractHornetMiddleware.APP_CONFIG;
        const dataPathPrefix = hornet_js_utils_1.Utils.buildContextPath(hornet_js_utils_1.Utils.config.getOrDefault("fullSpa.name", AbstractHornetMiddleware.APP_CONFIG.routesDataContext));
        super((req, res, next) => {
            callbacksStorage.bindEmitter(req);
            callbacksStorage.bindEmitter(res);
            callbacksStorage.run(() => {
                callbacksStorage.set("hornet.request", req);
                callbacksStorage.set("hornet.response", res);
                callbacksStorage.set("hornet.routeType", parseUrl.original(req).pathname.indexOf(dataPathPrefix) === 0
                    ? abstract_routes_1.RouteType.DATA
                    : abstract_routes_1.RouteType.PAGE);
                const currentUrl = parseUrl.original(req).pathname;
                const relativePathname = currentUrl.replace(hornet_js_utils_1.Utils.getContextPath(), "");
                callbacksStorage.set("hornet.routePath", relativePathname);
                callbacksStorage.set("hornet.currentUrl", currentUrl);
                if (appConfig.initCls) {
                    const initCls = appConfig.initCls.evaluateInitCls();
                    if (initCls) {
                        callbacksStorage.set("hornet.initCls", initCls);
                    }
                }
                next();
            });
        });
    }
}
exports.HornetContextInitializerMiddleware = HornetContextInitializerMiddleware;
class HornetContextInitializerSubMiddleware extends AbstractHornetSubMiddleware {
    constructor(config, prefix) {
        const dataPathPrefix = hornet_js_utils_1.Utils.buildContextPath("/" + prefix + (config
            || AbstractHornetMiddleware.APP_CONFIG).routesDataContext);
        super((req, res, next) => {
            hornet_js_utils_1.Utils.setCls("hornet.routeType", parseUrl.original(req).pathname.indexOf(dataPathPrefix) === 0
                ? abstract_routes_1.RouteType.DATA
                : abstract_routes_1.RouteType.PAGE);
            next();
        }, null, config);
    }
}
exports.HornetContextInitializerSubMiddleware = HornetContextInitializerSubMiddleware;
// ------------------------------------------------------------------------------------------------------------------- //
//                                      LoggerTIDMiddleware
// ------------------------------------------------------------------------------------------------------------------- //
const { v4: uuidv4 } = require('uuid');
class LoggerTIDMiddleware extends AbstractHornetMiddleware {
    constructor() {
        super((req, res, next) => {
            hornet_js_utils_1.Utils.getContinuationStorage().bind((req, res, next) => {
                hornet_js_utils_1.Utils.setCls("hornet.tid", uuidv4().substr(0, 8));
                timer_1.Timer.startTimer(timer_1.Timer.TIMER_REQUEST);
                timer_1.Timer.startTimer(timer_1.Timer.TIMER_SERVER_HTTP);
                next();
            })(req, res, next);
        });
    }
}
exports.LoggerTIDMiddleware = LoggerTIDMiddleware;
// ------------------------------------------------------------------------------------------------------------------- //
//                                      LoggerUserMiddleware
// ------------------------------------------------------------------------------------------------------------------- //
class LoggerUserMiddleware extends AbstractHornetMiddleware {
    constructor() {
        super((req, res, next) => {
            hornet_js_utils_1.Utils.setCls("hornet.user", req.user);
            next();
        });
    }
}
exports.LoggerUserMiddleware = LoggerUserMiddleware;
LoggerUserMiddleware.logger = logger_1.Logger.getLogger("hornet-js-core.middlewares.LoggerUserMiddleware");
// ------------------------------------------------------------------------------------------------------------------- //
//                                      WelcomePageRedirectMiddleware
// ------------------------------------------------------------------------------------------------------------------- //
const parseUrl = require("parseurl");
class WelcomePageRedirectMiddleware extends AbstractHornetMiddleware {
    constructor() {
        const contextPath = hornet_js_utils_1.Utils.buildContextPath("/").replace(/(^.*)([^\/]+)\/+$/g, "$1$2");
        const welcomePage = hornet_js_utils_1.Utils.buildContextPath(hornet_js_utils_1.Utils.appSharedProps.get("welcomePageUrl")).replace(/(^.*)([^\/]+)\/+$/g, "$1$2");
        super((req, res, next) => {
            const pathname = parseUrl.original(req).pathname.replace(/(^.*)([^\/]+)\/+$/g, "$1$2");
            if (pathname === contextPath && welcomePage !== contextPath) {
                return res.redirect(welcomePage);
            }
            else {
                next();
            }
        });
    }
}
exports.WelcomePageRedirectMiddleware = WelcomePageRedirectMiddleware;
WelcomePageRedirectMiddleware.logger = logger_1.Logger.getLogger("hornet-js-core.middlewares.WelcomePageRedirectMiddleware");
// ------------------------------------------------------------------------------------------------------------------- //
//                                      WithoutSlashPageRedirectMiddleware
// ------------------------------------------------------------------------------------------------------------------- //
class WithoutSlashPageRedirectMiddleware extends AbstractHornetMiddleware {
    constructor() {
        const contextPath = hornet_js_utils_1.Utils.getContextPath();
        super((req, res, next) => {
            const pathname = parseUrl.original(req).pathname;
            if (pathname === contextPath) {
                return res.redirect(contextPath + "/");
            }
            else {
                next();
            }
        });
    }
}
exports.WithoutSlashPageRedirectMiddleware = WithoutSlashPageRedirectMiddleware;
// ------------------------------------------------------------------------------------------------------------------- //
//                                      StaticNodeHttpHeaderMiddleware
// ------------------------------------------------------------------------------------------------------------------- //
class StaticNodeHttpHeaderMiddleware extends AbstractHornetMiddleware {
    constructor() {
        const staticUrl = hornet_js_utils_1.Utils.buildStaticPath("/").replace(hornet_js_utils_1.Utils.getContextPath(), "");
        super((req, res, next) => {
            res.setHeader("X-Static-From", "app");
            next();
        }, staticUrl);
    }
}
exports.StaticNodeHttpHeaderMiddleware = StaticNodeHttpHeaderMiddleware;
StaticNodeHttpHeaderMiddleware.logger = logger_1.Logger.getLogger("hornet-js-core.middlewares.StaticNodeHttpHeaderMiddleware");
// ------------------------------------------------------------------------------------------------------------------- //
//                                      StaticPathMiddleware
// ------------------------------------------------------------------------------------------------------------------- //
const express = require("express");
class StaticPathMiddleware extends AbstractHornetMiddleware {
    constructor() {
        const appConfig = AbstractHornetMiddleware.APP_CONFIG;
        const staticPath = path.join(appConfig.serverDir, appConfig.staticPath);
        const staticUrl = hornet_js_utils_1.Utils.buildStaticPath("/").replace(hornet_js_utils_1.Utils.getContextPath(), "");
        StaticPathMiddleware.logger.trace("Emplacement des ressources statiques '" + staticUrl + "' :", staticPath);
        super(express.static(staticPath, { fallthrough: false }), staticUrl);
    }
}
exports.StaticPathMiddleware = StaticPathMiddleware;
StaticPathMiddleware.logger = logger_1.Logger.getLogger("hornet-js-core.middlewares.StaticPathMiddleware");
// ------------------------------------------------------------------------------------------------------------------- //
//                                      StaticPathErrorMiddleware
// ------------------------------------------------------------------------------------------------------------------- //
class StaticPathErrorMiddleware extends AbstractHornetMiddleware {
    constructor() {
        const staticUrl = hornet_js_utils_1.Utils.buildStaticPath("/").replace(hornet_js_utils_1.Utils.getContextPath(), "");
        super((err, req, res, next) => {
            res.status(404).send("Resource not found").end();
            StaticPathErrorMiddleware.logger.error("Ressource statique non trouvée", err);
        }, staticUrl);
    }
}
exports.StaticPathErrorMiddleware = StaticPathErrorMiddleware;
StaticPathErrorMiddleware.logger = logger_1.Logger.getLogger("hornet-js-core.middlewares.StaticPathErrorMiddleware");
// ------------------------------------------------------------------------------------------------------------------- //
//                                      BodyParserJsonMiddleware
// ------------------------------------------------------------------------------------------------------------------- //
const bodyParser = require("body-parser");
class BodyParserJsonMiddleware extends AbstractHornetMiddleware {
    constructor() {
        super(bodyParser.json({ limit: hornet_js_utils_1.Utils.config.getOrDefault("server.bodyParserLimit", "50mb") }));
    }
}
exports.BodyParserJsonMiddleware = BodyParserJsonMiddleware;
// ------------------------------------------------------------------------------------------------------------------- //
//                                      BodyParserUrlEncodedMiddleware
// ------------------------------------------------------------------------------------------------------------------- //
class BodyParserUrlEncodedMiddleware extends AbstractHornetMiddleware {
    constructor() {
        super(bodyParser.urlencoded({
            extended: true,
            limit: hornet_js_utils_1.Utils.config.getOrDefault("server.bodyParserLimit", "50mb"),
        }));
    }
}
exports.BodyParserUrlEncodedMiddleware = BodyParserUrlEncodedMiddleware;
// ------------------------------------------------------------------------------------------------------------------- //
//                                      SessionMiddleware
// ------------------------------------------------------------------------------------------------------------------- //
const session_manager_1 = require("hornet-js-core/src/session/session-manager");
class SessionMiddleware extends AbstractHornetMiddleware {
    constructor() {
        const sessionConfiguration = {
            trustProxy: hornet_js_utils_1.Utils.config.getOrDefault("server.trustProxy", false),
            store: AbstractHornetMiddleware.APP_CONFIG.sessionStore || null,
            // secret: Utils.config.get('cookie.secret'),
            secret: null,
            cookie: {
                route: hornet_js_utils_1.Utils.config.getOrDefault("server.route", null),
                domain: hornet_js_utils_1.Utils.config.getOrDefault("cookie.domain", null),
                path: hornet_js_utils_1.Utils.config.getOrDefault("cookie.path", hornet_js_utils_1.Utils.buildContextPath("/")),
                httpOnly: hornet_js_utils_1.Utils.config.getOrDefault("cookie.httpOnly", true),
                secure: (isEnvProduction && hornet_js_utils_1.Utils.config.getOrDefault("cookie.secure", false)),
                maxAge: hornet_js_utils_1.Utils.config.getOrDefault("cookie.maxAge", undefined),
                sameSite: hornet_js_utils_1.Utils.config.getOrDefault("cookie.sameSite", undefined),
            },
            sessionTimeout: hornet_js_utils_1.Utils.config.getOrDefault("server.sessionTimeout", 1800000),
            alwaysSetCookie: hornet_js_utils_1.Utils.config.getOrDefault("cookie.alwaysSetCookie", false),
            saveUnmodifiedSession: null, // laisse le session-manager décider en fonction du type de store
        };
        super(session_manager_1.SessionManager.middleware(sessionConfiguration));
    }
}
exports.SessionMiddleware = SessionMiddleware;
// ------------------------------------------------------------------------------------------------------------------- //
//                                      MulterMiddleware
// ------------------------------------------------------------------------------------------------------------------- //
const multer = require("multer");
const custom_store_engine_1 = require("hornet-js-core/src/upload/custom-store-engine");
class MulterMiddleware extends AbstractHornetMiddleware {
    constructor() {
        const option = MulterMiddleware.defaultOption;
        if (MulterMiddleware.option) {
            merge(option, MulterMiddleware.option);
        }
        super(multer(option).any());
    }
}
exports.MulterMiddleware = MulterMiddleware;
/**
 * Parametrage static de middleware multer
 */
MulterMiddleware.option = undefined;
MulterMiddleware.defaultOption = {
    storage: hornet_js_utils_1.Utils.config.getOrDefault("server.uploadAntivirus", false) ?
        new custom_store_engine_1.CustomStoreEngine()
        : multer.memoryStorage(),
    limits: {
        fileSize: hornet_js_utils_1.Utils.config.getOrDefault("server.uploadFileSize", 10000),
    },
};
// ------------------------------------------------------------------------------------------------------------------- //
//                                      SecurityMiddleware
// ------------------------------------------------------------------------------------------------------------------- //
const hpp = require("hpp");
const helmet = require("helmet");
class SecurityMiddleware extends AbstractHornetMiddleware {
    insertMiddleware(app) {
        if (isSecurityEnabled(isEnvProduction, SecurityMiddleware.logger)) {
            // Enlève le header 'x-powered-by' pour éviter de montrer que le site est généré par express
            app.disable("x-powered-by");
            this.hppConfiguration(app);
            this.ieNoOpenConfiguration(app);
            this.noSniffConfiguration(app);
            this.cspConfiguration(app);
            this.frameguardConfiguration(app);
            this.xssFilterConfiguration(app);
            this.hpkpConfiguration(app);
            this.hstsConfiguration(app);
            this.referrerPolicyConfiguration(app);
        }
    }
    hppConfiguration(app) {
        // Suppression des doublons de paramètres GET
        if (checkSecurityConfiguration("security.hpp", false, "HPP 'HTTP Parameter Pollution'", SecurityMiddleware.logger)) {
            app.use(hpp());
        }
    }
    ieNoOpenConfiguration(app) {
        if (checkSecurityConfiguration("security.ieNoOpen", false, "ienoopen 'IE, restrict untrusted HTML: ieNoOpen'", SecurityMiddleware.logger)) {
            // Pour IE8+: ajout le header X-Download-Options
            app.use(helmet.ieNoOpen());
        }
    }
    noSniffConfiguration(app) {
        if (checkSecurityConfiguration("security.noSniff", false, "noSniff 'Don't infer the MIME type: noSniff'", SecurityMiddleware.logger)) {
            // Pour IE8+: ajout le header X-Download-Options
            app.use(helmet.noSniff());
        }
    }
    cspConfiguration(app) {
        // Ajout des headers "content security" pour empêcher le chargement de scripts venant d'autres domaines
        if (checkSecurityConfiguration("security.csp", true, "CSP 'Content Security Policy'", SecurityMiddleware.logger)) {
            const directives = {
                // defaultSrc: Utils.config.getOrDefault("security.csp.defaultSrc", ["'self'", "'unsafe-inline'", "'unsafe-eval'"]),
                baseUri: hornet_js_utils_1.Utils.config.getOrDefault("security.csp.baseUri", ["'self'"]),
                childSrc: hornet_js_utils_1.Utils.config.getOrDefault("security.csp.childSrc", ["'self'"]),
                connectSrc: hornet_js_utils_1.Utils.config.getOrDefault("security.csp.connectSrc", ["'self'"]),
                fontSrc: hornet_js_utils_1.Utils.config.getOrDefault("security.csp.fontSrc", ["'self'"]),
                formAction: hornet_js_utils_1.Utils.config.getOrDefault("security.csp.formAction", ["'self'"]),
                frameAncestors: hornet_js_utils_1.Utils.config.getOrDefault("security.csp.frameAncestors", ["'self'"]),
                frameSrc: hornet_js_utils_1.Utils.config.getOrDefault("security.csp.frameSrc", ["'self'"]),
                imgSrc: hornet_js_utils_1.Utils.config.getOrDefault("security.csp.imgSrc", ["'self'"]),
                manifestSrc: hornet_js_utils_1.Utils.config.getOrDefault("security.csp.manifestSrc", ["'self'"]),
                mediaSrc: hornet_js_utils_1.Utils.config.getOrDefault("security.csp.mediaSrc", ["'self'"]),
                objectSrc: hornet_js_utils_1.Utils.config.getOrDefault("security.csp.objectSrc", ["'self'"]),
                // todo deal with missing directives
                // reflectedXss: Utils.config.getOrDefault("security.csp.reflectedXss", "block"),
                // todo create post route to report policy failures
                // reportUri: Utils.config.getOrDefault("security.csp.reportUri", ""),
                scriptSrc: hornet_js_utils_1.Utils.config.getOrDefault("security.csp.scriptSrc", ["'self'", "'unsafe-inline'", "'unsafe-eval'"]),
                styleSrc: hornet_js_utils_1.Utils.config.getOrDefault("security.csp.styleSrc", ["'self'", "'unsafe-inline'"]),
            };
            const sandbox = hornet_js_utils_1.Utils.config.getOrDefault("security.csp.sandbox", false); // TODO A traiter spécifiquement
            if (sandbox) {
                directives.sandbox = sandbox;
            }
            const pluginTypes = hornet_js_utils_1.Utils.config.getOrDefault("security.csp.pluginTypes", false);
            if (pluginTypes) {
                directives.pluginTypes = pluginTypes;
            }
            const blockAllMixedContent = hornet_js_utils_1.Utils.config.getOrDefault("security.csp.blockAllMixedContent", false);
            if (blockAllMixedContent) {
                directives.blockAllMixedContent = blockAllMixedContent;
            }
            const upgradeInsecureRequests = hornet_js_utils_1.Utils.config.getOrDefault("security.csp.upgradeInsecureRequests", false);
            if (upgradeInsecureRequests) {
                directives.upgradeInsecureRequests = upgradeInsecureRequests;
            }
            app.use(helmet.contentSecurityPolicy({
                directives,
                disableAndroid: hornet_js_utils_1.Utils.config.getOrDefault("security.csp.disableAndroid", false),
                reportOnly: hornet_js_utils_1.Utils.config.getOrDefault("security.csp.reportOnly", false),
                setAllHeaders: hornet_js_utils_1.Utils.config.getOrDefault("security.csp.setAllHeaders", true),
            }));
        }
    }
    referrerPolicyConfiguration(app) {
        app.use(helmet.referrerPolicy({
            policy: hornet_js_utils_1.Utils.config.getOrDefault("security.csp.referrer", "origin-when-cross-origin"),
        }));
    }
    frameguardConfiguration(app) {
        // Pour empêcher la mise en itiframe
        if (checkSecurityConfiguration("security.frameguard", true, "frameguard", SecurityMiddleware.logger)) {
            const action = hornet_js_utils_1.Utils.config.getOrDefault("security.frameguard.action", "deny");
            let domain = undefined;
            if (action === "allow-from") {
                domain = hornet_js_utils_1.Utils.config.getOrDefault("security.frameguard.domain", "'self'");
            }
            app.use(helmet.frameguard({
                action,
                domain,
            }));
        }
    }
    xssFilterConfiguration(app) {
        if (checkSecurityConfiguration("security.xss", true, "XssFilter 'Cross-site scripting Filter'", SecurityMiddleware.logger)) {
            // Ajoute le header X-XSS-Protection pour essayer de protéger contre des attaques XSS simples
            // This has some security problems for old IE!
            app.use(helmet.xssFilter({ setOnOldIE: hornet_js_utils_1.Utils.config.getOrDefault("security.xss.setOnOldIE", true) }));
        }
    }
    hpkpConfiguration(app) {
        // Ajoute les headers HTTP Public Key Pinning pour sécuriser les connexions SSL
        if (checkSecurityConfiguration("security.hpkp", true, "HPKP 'HTTP Public Key Pinning'", SecurityMiddleware.logger)) {
            app.use(helmet.hpkp({
                maxAge: hornet_js_utils_1.Utils.config.getOrDefault("security.hpkp.maxAge", 7776000000),
                sha256s: hornet_js_utils_1.Utils.config.get("security.hpkp.sha256s"),
                includeSubdomains: hornet_js_utils_1.Utils.config.getOrDefault("security.hpkp.includeSubdomains", true),
                reportUri: hornet_js_utils_1.Utils.config.getOrDefault("security.hpkp.reportUri", null),
                reportOnly: hornet_js_utils_1.Utils.config.getOrDefault("security.hpkp.reportOnly", false),
            }));
        }
    }
    hstsConfiguration(app) {
        if (checkSecurityConfiguration("security.hsts", true, "HSTS 'HTTP Strict-Transport-Security'", SecurityMiddleware.logger)) {
            const hstsMiddleware = helmet.hsts({
                // Must be at least 18 weeks to be approved by Google
                maxAge: hornet_js_utils_1.Utils.config.getOrDefault("security.hsts.maxAge", 10886400000),
                // Must be enabled to be approved by Google
                includeSubDomains: hornet_js_utils_1.Utils.config.getOrDefault("security.hsts.includeSubDomains", true),
                preload: hornet_js_utils_1.Utils.config.getOrDefault("security.hsts.preload", false),
            });
            app.use((req, res, next) => {
                if (req.secure) {
                    // Ajoute le header Strict-Transport-Security pour demander au navigateur client un accès au site en https
                    hstsMiddleware(req, res, next);
                }
                else {
                    next();
                }
            });
        }
    }
}
exports.SecurityMiddleware = SecurityMiddleware;
SecurityMiddleware.logger = logger_1.Logger.getLogger("hornet-js-core.middlewares.SecurityMiddleware");
class CsrfMiddleware extends AbstractHornetMiddleware {
    insertMiddleware(app) {
        if (isSecurityEnabled(isEnvProduction, CsrfMiddleware.logger)) {
            this.csrfConfiguration(app);
        }
    }
    csrfConfiguration(app) {
        if (checkSecurityConfiguration("security.csrf", true, "CSRF 'Cross-Site Request Forgery'", CsrfMiddleware.logger)) {
            app.use(CsrfMiddleware.middleware);
        }
    }
    /**
     * Retourne un nouveau token aléatoire
     * @return {string}
     */
    static generateToken() {
        const newToken = require('crypto').randomBytes(8).toString('base64');
        CsrfMiddleware.logger.trace("Génération d'un token CSRF:", newToken);
        return newToken;
    }
    /**
     * Middleware express pour sécuriser les verbs HTTP PUT, POST, PATCH et DELETE d'attaques CSRF
     * @param req
     * @param resrouter
     * @param next
     */
    static middleware(req, res, next) {
        const routesAuthorization = hornet_js_utils_1.Utils.hasCls() && hornet_js_utils_1.Utils.getCls("hornet.routeAuthorization");
        if (req.method === "PUT" || (req.method === "POST" && (!routesAuthorization || routesAuthorization.length > 0)) || req.method === "PATCH" || req.method === "DELETE") {
            let incomingCsrf = req.headers[CsrfMiddleware.HEADER_CSRF_NAME];
            if (isUndefined(incomingCsrf)) {
                CsrfMiddleware.logger.trace(CsrfMiddleware.HEADER_CSRF_NAME, "non present, recherche dans le body de la requête");
                incomingCsrf = req.body && req.body[CsrfMiddleware.HEADER_CSRF_NAME];
            }
            const sessionCsrf = req.getSession().getAttribute(CsrfMiddleware.CSRF_SESSION_KEY);
            const valid = CsrfMiddleware.isTokenValid(incomingCsrf, sessionCsrf);
            if (valid) {
                // Le token est correcte, l'accès est autorisé
                next();
            }
            else {
                CsrfMiddleware.logger.debug("Csrf mismatch, retourning 417 status");
                res.status(417);
                res.end();
            }
        }
        else {
            // Pas de sécurité csrf sur ce verb HTTP.
            // On récupère le token courant en session, généré si absent, puis on l'insère dans le CLS
            let token = req.getSession().getAttribute(CsrfMiddleware.CSRF_SESSION_KEY);
            if (token != null) {
                CsrfMiddleware.logger.debug("Token CSRF already exists in session");
            }
            else {
                token = CsrfMiddleware.generateToken();
                req.getSession().setAttribute(CsrfMiddleware.CSRF_SESSION_KEY, token);
            }
            hornet_js_utils_1.Utils.setCls("hornet.csrf", token);
            next();
        }
    }
    /**
     * Détermine si le token envoyé correspond à celui stocké en session
     * @param incomingCsrf
     * @param sessionCsrf
     * @return {boolean}
     */
    static isTokenValid(incomingCsrf, sessionCsrf) {
        CsrfMiddleware.logger.trace("incomingCsrf =", incomingCsrf, "sessionCsrf =", sessionCsrf);
        return incomingCsrf === sessionCsrf;
    }
}
exports.CsrfMiddleware = CsrfMiddleware;
CsrfMiddleware.logger = logger_1.Logger.getLogger("hornet-js-core.middlewares.CsrfMiddleware");
CsrfMiddleware.HEADER_CSRF_NAME = "x-csrf-token";
CsrfMiddleware.CSRF_SESSION_KEY = "security.csrf";
// ------------------------------------------------------------------------------------------------------------------- //
//                                      InternationalizationMiddleware                                                 //
// ------------------------------------------------------------------------------------------------------------------- //
const abstract_i18n_loader_1 = require("hornet-js-core/src/i18n/abstract-i18n-loader");
const cookie_manager_1 = require("hornet-js-core/src/session/cookie-manager");
class InternationalizationMiddleware extends AbstractHornetMiddleware {
    constructor(config) {
        super((req, res, next) => {
            const appConfig = this.config || AbstractHornetMiddleware.APP_CONFIG;
            const isI18nInSession = req.session && req.session.i18n;
            let localeI18n = (isI18nInSession) ? req.session.i18n : hornet_js_utils_1.Utils.config.getOrDefault("localeI18n", {});
            if (!isI18nInSession) {
                const cookLocaleI18n = cookie_manager_1.CookieManager.getCookie(req, "internationalization");
                if (cookLocaleI18n !== undefined) {
                    const shortLang = cookLocaleI18n.split("-");
                    localeI18n = { locale: cookLocaleI18n, lang: shortLang[1] };
                }
            }
            if (appConfig.internationalization instanceof abstract_i18n_loader_1.AbstractI18nLoader || (appConfig.internationalization.getMessages && typeof appConfig.internationalization.getMessages === "function")) {
                hornet_js_utils_1.Utils.setCls("hornet.internationalization", appConfig.internationalization.getMessages(localeI18n));
            }
            if (!isI18nInSession) {
                InternationalizationMiddleware.logger.trace("InternationalizationMiddleware :  Mise en session de la locale "
                    + localeI18n.locale);
                if (!req.session) {
                    req.session = {};
                }
                req.session.i18n = localeI18n;
            }
            next();
        }, null, config);
    }
}
exports.InternationalizationMiddleware = InternationalizationMiddleware;
InternationalizationMiddleware.logger = logger_1.Logger.getLogger("hornet-js-core.middlewares.InternationalizationMiddleware");
// ------------------------------------------------------------------------------------------------------------------- //
//                                      ChangeI18nLocaleMiddleware
// ------------------------------------------------------------------------------------------------------------------- //
class ChangeI18nLocaleMiddleware extends AbstractHornetMiddleware {
    constructor() {
        const appConfig = AbstractHornetMiddleware.APP_CONFIG;
        if (!hornet_js_utils_1.Utils.appSharedProps.get("listLanguage") && appConfig.internationalization) {
            hornet_js_utils_1.Utils.appSharedProps.set("listLanguage", appConfig.internationalization.getLocales());
        }
        super((req, res, next) => {
            if (some(appConfig.internationalization.getLocales(), { locale: req.params.i18n })) {
                ChangeI18nLocaleMiddleware.logger.trace("ChangeI18nLocaleMiddleware :  Changement de la locale " + req.params.i18n);
                const shortLang = req.params.i18n.split("-");
                req.session.i18n = { locale: req.params.i18n, lang: shortLang[1] };
                const i18n = appConfig.internationalization.getMessages(req.session.i18n);
                hornet_js_utils_1.Utils.setCls("hornet.internationalization", i18n);
                const cookiesI18nOption = {
                    httpOnly: hornet_js_utils_1.Utils.config.getOrDefault("cookie.httpOnly", true),
                    secure: (isEnvProduction && hornet_js_utils_1.Utils.config.getOrDefault("cookie.secure", false)),
                    maxAge: hornet_js_utils_1.Utils.config.getOrDefault("cookie.maxAge", undefined),
                    sameSite: hornet_js_utils_1.Utils.config.getOrDefault("cookie.sameSite", undefined)
                };
                cookie_manager_1.CookieManager.setCookie(res, "internationalization", req.params.i18n, cookiesI18nOption);
                res.type("application/json");
                res.status(200).send(i18n);
                res.end();
            }
            else {
                ChangeI18nLocaleMiddleware.logger.trace("ChangeI18nLocaleMiddleware :  La locale demandée "
                    + req.params.i18n + "n'existe pas");
                const err = new technical_error_1.TechnicalError("ERR_TECH_UNKNOWN", {
                    errorMessage: "La locale '" + req.params.i18n + "' n\'existe pas",
                    httpStatus: 200,
                });
                res.json(service_api_results_1.NodeApiResultBuilder.buildError(err));
                res.end();
            }
        });
        this.prefix = "/changeLanguage/:i18n";
    }
}
exports.ChangeI18nLocaleMiddleware = ChangeI18nLocaleMiddleware;
ChangeI18nLocaleMiddleware.logger = logger_1.Logger.getLogger("hornet-js-core.middlewares.ChangeI18nLocaleMiddleware");
// ------------------------------------------------------------------------------------------------------------------- //
//                                      SetExpandedLayoutMiddleware
// ------------------------------------------------------------------------------------------------------------------- //
class SetExpandedLayoutMiddleware extends AbstractHornetMiddleware {
    constructor() {
        super((req, res, next) => {
            SetExpandedLayoutMiddleware.logger.trace("SetLayoutMiddleware : set isExpandedLayout appSharedProps with value:", req.body.isExpandedLayout);
            hornet_js_utils_1.Utils.setCls("hornet.expandedLayout", req.body.isExpandedLayout);
            const cookiesExpandedLayoutOption = {
                httpOnly: hornet_js_utils_1.Utils.config.getOrDefault("cookie.httpOnly", true),
                secure: (isEnvProduction && hornet_js_utils_1.Utils.config.getOrDefault("cookie.secure", false)),
                maxAge: hornet_js_utils_1.Utils.config.getOrDefault("cookie.maxAge", undefined),
                sameSite: hornet_js_utils_1.Utils.config.getOrDefault("cookie.sameSite", undefined)
            };
            cookie_manager_1.CookieManager.setCookie(res, "expandedLayout", req.body.isExpandedLayout, cookiesExpandedLayoutOption);
            res.type("application/json");
            res.status(200).send({ isExpandedLayout: req.body.isExpandedLayout });
            res.end();
        });
        this.prefix = "/setExpandedLayout";
    }
}
exports.SetExpandedLayoutMiddleware = SetExpandedLayoutMiddleware;
SetExpandedLayoutMiddleware.logger = logger_1.Logger.getLogger("hornet-js-core.middlewares.SetExpandedLayoutMiddleware");
// ------------------------------------------------------------------------------------------------------------------- //
//                                      IsExpandedLayoutMiddleware
// ------------------------------------------------------------------------------------------------------------------- //
class IsExpandedLayoutMiddleware extends AbstractHornetMiddleware {
    constructor() {
        super((req, res, next) => {
            const isExpandedLayout = cookie_manager_1.CookieManager.getCookie(req, "expandedLayout");
            IsExpandedLayoutMiddleware.logger.trace("IsExpandedLayoutMiddleware : get isExpandedLayout appSharedProps: ", isExpandedLayout);
            res.type("application/json");
            res.set("Cache-Control", "no-cache");
            res.status(200).send({ isExpandedLayout });
            res.end();
        });
        this.prefix = "/isExpandedLayout";
    }
}
exports.IsExpandedLayoutMiddleware = IsExpandedLayoutMiddleware;
IsExpandedLayoutMiddleware.logger = logger_1.Logger.getLogger("hornet-js-core.middlewares.IsExpandedLayoutMiddleware");
// ------------------------------------------------------------------------------------------------------------------- //
//                                      ExpandedLayoutHandlerMiddleware
// ------------------------------------------------------------------------------------------------------------------- //
class ExpandedLayoutHandlerMiddleware extends AbstractHornetMiddleware {
    constructor(config) {
        super((req, res, next) => {
            const isExpandedLayout = cookie_manager_1.CookieManager.getCookie(req, "expandedLayout");
            hornet_js_utils_1.Utils.setCls("hornet.expandedLayout", isExpandedLayout == "true");
            ExpandedLayoutHandlerMiddleware.logger.trace("ExpandedLayoutHandlerMiddleware valeur actuelle", isExpandedLayout);
            next();
        }, null, config);
    }
}
exports.ExpandedLayoutHandlerMiddleware = ExpandedLayoutHandlerMiddleware;
ExpandedLayoutHandlerMiddleware.logger = logger_1.Logger.getLogger("hornet-js-core.middlewares.ExpandedLayoutHandlerMiddleware");
// ------------------------------------------------------------------------------------------------------------------- //
//                                      wakeUpNodeSessionMiddleware
// ------------------------------------------------------------------------------------------------------------------- //
class wakeUpNodeSessionMiddleware extends AbstractHornetMiddleware {
    constructor() {
        super((req, res, next) => {
            wakeUpNodeSessionMiddleware.logger.trace("sollicitation du nodeJs suite à notification fin de session node");
            res.type("application/json");
            res.set("Cache-Control", "no-cache");
            res.status(200).send({});
            res.end();
        });
        this.prefix = "/wakeUpNodeSession";
    }
}
exports.wakeUpNodeSessionMiddleware = wakeUpNodeSessionMiddleware;
wakeUpNodeSessionMiddleware.logger = logger_1.Logger.getLogger("hornet-js-core.middlewares.wakeUpNodeSessionMiddleware");
// ------------------------------------------------------------------------------------------------------------------- //
//                                      RouterServerMiddleware                                                         //
// ------------------------------------------------------------------------------------------------------------------- //
const router_server_1 = require("hornet-js-core/src/routes/router-server");
class RouterServerMiddleware extends AbstractHornetMiddleware {
    constructor(config) {
        super(null, null, config);
        const appConfig = this.config || AbstractHornetMiddleware.APP_CONFIG;
        this.router = new router_server_1.RouterServer(appConfig.defaultRoutesClass, appConfig.routesLoaderfn, appConfig.routesLoaderPaths, appConfig.routesDataContext);
    }
    insertMiddleware(app) {
        app.use(this.router.pageMiddleware());
        app.use(hornet_js_utils_1.Utils.config.getOrDefault("fullSpa.name", AbstractHornetMiddleware.APP_CONFIG.routesDataContext), this.router.dataMiddleware());
    }
}
exports.RouterServerMiddleware = RouterServerMiddleware;
class RouterServerSubMiddleware extends AbstractHornetSubMiddleware {
    constructor(config) {
        super(null, null, config);
        const appConfig = this.config || AbstractHornetMiddleware.APP_CONFIG;
        this.router = new router_server_1.RouterServer(appConfig.defaultRoutesClass, appConfig.routesLoaderfn, appConfig.routesLoaderPaths, appConfig.routesDataContext);
    }
    insertRouterMiddleware(router) {
        router.use(this.router.pageMiddleware());
        router.use(hornet_js_utils_1.Utils.config.getOrDefault("fullSpa.name", (this.config || AbstractHornetMiddleware.APP_CONFIG).routesDataContext), this.router.dataMiddleware());
    }
}
exports.RouterServerSubMiddleware = RouterServerSubMiddleware;
// ------------------------------------------------------------------------------------------------------------------- //
//                                      UserAccessSecurityMiddleware
// ------------------------------------------------------------------------------------------------------------------- //
const authentication_utils_1 = require("hornet-js-utils/src/authentication-utils");
const security_error_1 = require("hornet-js-utils/src/exception/security-error");
const not_found_error_1 = require("hornet-js-utils/src/exception/not-found-error");
class UserAccessSecurityMiddleware extends AbstractHornetMiddleware {
    constructor() {
        super((req, res, next) => {
            const routesAuthorization = hornet_js_utils_1.Utils.getCls("hornet.routeAuthorization");
            if (!authentication_utils_1.AuthUtils.isAllowed(req.user, routesAuthorization)) {
                let e;
                if (isEnvProduction) { // best practices (security)
                    e = new not_found_error_1.NotFoundError();
                }
                else {
                    e = new security_error_1.SecurityError();
                }
                UserAccessSecurityMiddleware.logger.trace("Tentative d'accès à une route non autorisée", req.url, e.reportId);
                throw e;
            }
            next();
        });
    }
}
exports.UserAccessSecurityMiddleware = UserAccessSecurityMiddleware;
UserAccessSecurityMiddleware.logger = logger_1.Logger.getLogger("hornet-js-core.middlewares.UserAccessSecurityMiddleware");
// ------------------------------------------------------------------------------------------------------------------- //
//                                      MenuConfigMiddleware
// ------------------------------------------------------------------------------------------------------------------- //
class MenuConfigMiddleware extends AbstractHornetMiddleware {
    constructor() {
        super((req, res, next) => {
            hornet_js_utils_1.Utils.setCls("hornet.menuConfig", [...AbstractHornetMiddleware.APP_CONFIG.menuConfig]);
            next();
        });
    }
}
exports.MenuConfigMiddleware = MenuConfigMiddleware;
const async_executor_1 = require("hornet-js-core/src/executor/async-executor");
const async_element_1 = require("hornet-js-core/src/executor/async-element");
const service_api_results_1 = require("hornet-js-core/src/services/service-api-results");
const validation_error_1 = require("hornet-js-utils/src/exception/validation-error");
const hornet_result_1 = require("hornet-js-core/src/result/hornet-result");
const result_stream_1 = require("hornet-js-core/src/result/result-stream");
const result_json_1 = require("hornet-js-core/src/result/result-json");
class DataRenderingMiddleware extends AbstractHornetMiddleware {
    constructor() {
        super((req, res, next) => {
            try {
                const routeInfos = hornet_js_utils_1.Utils.getCls("hornet.routeInfos");
                // route de type 'DATA' uniquement
                if (hornet_js_utils_1.Utils.getCls("hornet.routeType") === abstract_routes_1.RouteType.DATA) {
                    const dataRouteInfos = routeInfos;
                    if (!dataRouteInfos) {
                        const mess = "DataRouteInfos inexistant pour l'url : " + req.originalUrl;
                        DataRenderingMiddleware.logger.warn(mess);
                        throw new technical_error_1.TechnicalError("ERR_TECH_UNKNOWN", { errorMessage: mess, httpStatus: 200 });
                    }
                    else {
                        const executor = new async_executor_1.AsyncExecutor();
                        executor.addElement(new async_element_1.AsyncElement(hornet_js_utils_1.Utils.getContinuationStorage().bind((next) => {
                            timer_1.Timer.startTimer(timer_1.Timer.TIMER_ACTION, timer_1.Timer.TIMER_SERVER_HTTP);
                            const action = new (dataRouteInfos.getAction())();
                            action.req = req;
                            action.res = res;
                            action.attributes = routeInfos.getAttributes();
                            const actionService = dataRouteInfos.getService();
                            if (actionService) {
                                if (!actionService.prototype) {
                                    action.service = actionService;
                                }
                                else {
                                    DataRenderingMiddleware.logger.deprecated("Les services doivent être instanciés afin de pouvoir être utilisés par le composant DataRenderingMiddleware. Les services non instanciés ne seront bientôt plus supportés. Voir l'utilisation de l'injector avec un scope SINGLETON par exemple ou l'instanciation dans la déclaration de vos routes");
                                    action.service = new actionService();
                                }
                            }
                            const validator = action.getDataValidator();
                            if (validator) {
                                const data = cloneDeep(action.getPayload());
                                const validationRes = validator.validate(data);
                                if (!validationRes.valid) {
                                    DataRenderingMiddleware.logger.warn("Données invalides (la validation aurait dû être effectuée côté client) : ", validationRes.errors);
                                    let validationError = new validation_error_1.ValidationError(undefined, hornet_js_utils_1.Utils.config.getOrDefault("server.rethrow", false) ? { httpStatus: 400 } : {});
                                    if (hornet_js_utils_1.Utils.config.getOrDefault("server.rethrowValidationErrors", false)) {
                                        validationError = new validation_error_1.ValidationError(validationRes, hornet_js_utils_1.Utils.config.getOrDefault("server.rethrow", false) ? { httpStatus: 400 } : {});
                                    }
                                    throw validationError;
                                }
                            }
                            const exec = action.execute();
                            exec.then(hornet_js_utils_1.Utils.getContinuationStorage().bind((result) => {
                                timer_1.Timer.startTimer(timer_1.Timer.TIMER_SERVER_HTTP, timer_1.Timer.TIMER_ACTION);
                                const newResult = (result instanceof hornet_result_1.HornetResult) ? result : new result_json_1.ResultJSON({ data: result });
                                if (!(newResult instanceof result_stream_1.ResultStream)) {
                                    res.status(newResult.status);
                                }
                                return newResult.manageResponse(res, req);
                            })).then(hornet_js_utils_1.Utils.getContinuationStorage().bind((send) => {
                                timer_1.Timer.stopTimer(timer_1.Timer.TIMER_SERVER_HTTP);
                                timer_1.Timer.stopTimer(timer_1.Timer.TIMER_REQUEST);
                                timer_1.Timer.logFinally("global", req.originalUrl, req.method, res.statusCode && res.statusCode.toString());
                                if (send) {
                                    res.end();
                                }
                            })).catch(hornet_js_utils_1.Utils.getContinuationStorage().bind((error) => {
                                timer_1.Timer.startTimer(timer_1.Timer.TIMER_SERVER_HTTP, timer_1.Timer.TIMER_ACTION);
                                DataRenderingMiddleware.logger.error("Erreur de service..." + error);
                                next(error);
                            }));
                        })));
                        executor.on("end", (err) => {
                            if (err) {
                                next(err);
                            }
                        });
                        executor.execute();
                    }
                }
            }
            catch (e) {
                next(e);
            }
        });
    }
}
exports.DataRenderingMiddleware = DataRenderingMiddleware;
DataRenderingMiddleware.logger = logger_1.Logger.getLogger("hornet-js-core.middlewares.DataRenderingMiddleware");
class DataRenderingSubMiddleware extends AbstractHornetSubMiddleware {
    constructor() {
        super((req, res, next) => {
            try {
                const routeInfos = hornet_js_utils_1.Utils.getCls("hornet.routeInfos");
                // route de type 'DATA' uniquement
                if (hornet_js_utils_1.Utils.getCls("hornet.routeType") === abstract_routes_1.RouteType.DATA) {
                    const dataRouteInfos = routeInfos;
                    if (!dataRouteInfos) {
                        const mess = "DataRouteInfos inexistant pour l'url : " + req.originalUrl;
                        DataRenderingSubMiddleware.logger.warn(mess);
                        throw new technical_error_1.TechnicalError("ERR_TECH_UNKNOWN", { errorMessage: mess, httpStatus: 200 });
                    }
                    else {
                        const executor = new async_executor_1.AsyncExecutor();
                        executor.addElement(new async_element_1.AsyncElement(hornet_js_utils_1.Utils.getContinuationStorage().bind((next) => {
                            timer_1.Timer.startTimer(timer_1.Timer.TIMER_ACTION, timer_1.Timer.TIMER_SERVER_HTTP);
                            const action = new (dataRouteInfos.getAction())();
                            action.req = req;
                            action.res = res;
                            action.attributes = routeInfos.getAttributes();
                            const actionService = dataRouteInfos.getService();
                            if (actionService) {
                                if (!actionService.prototype) {
                                    action.service = actionService;
                                }
                                else {
                                    DataRenderingSubMiddleware.logger.deprecated("Les services doivent être instanciés afin de pouvoir être utilisés par le composant DataRenderingSubMiddleware. Les services non instanciés ne seront bientôt plus supportés. Voir l'utilisation de l'injector avec un scope SINGLETON par exemple ou l'instanciation dans la déclaration de vos routes");
                                    action.service = new actionService();
                                }
                            }
                            const validator = action.getDataValidator();
                            if (validator) {
                                const data = cloneDeep(action.getPayload());
                                const validationRes = validator.validate(data);
                                if (!validationRes.valid) {
                                    DataRenderingSubMiddleware.logger.warn("Données invalides (la validation aurait dû être effectuée côté client) : ", validationRes.errors);
                                    throw new validation_error_1.ValidationError();
                                }
                            }
                            const exec = action.execute();
                            exec.then(hornet_js_utils_1.Utils.getContinuationStorage().bind((result) => {
                                timer_1.Timer.startTimer(timer_1.Timer.TIMER_SERVER_HTTP, timer_1.Timer.TIMER_ACTION);
                                const newResult = (result instanceof hornet_result_1.HornetResult)
                                    ? result
                                    : new result_json_1.ResultJSON({ data: result });
                                if (!(newResult instanceof result_stream_1.ResultStream)) {
                                    res.status(newResult.status);
                                }
                                return newResult.manageResponse(res, req);
                            })).then(hornet_js_utils_1.Utils.getContinuationStorage().bind((send) => {
                                timer_1.Timer.stopTimer(timer_1.Timer.TIMER_SERVER_HTTP);
                                timer_1.Timer.stopTimer(timer_1.Timer.TIMER_REQUEST);
                                timer_1.Timer.logFinally("global", req.originalUrl, req.method, res.statusCode && res.statusCode.toString());
                                if (send) {
                                    res.end();
                                }
                            })).catch(hornet_js_utils_1.Utils.getContinuationStorage().bind((error) => {
                                timer_1.Timer.startTimer(timer_1.Timer.TIMER_SERVER_HTTP, timer_1.Timer.TIMER_ACTION);
                                DataRenderingSubMiddleware.logger.error("Erreur de service..." + error);
                                next(error);
                            }));
                        })));
                        executor.on("end", (err) => {
                            if (err) {
                                next(err);
                            }
                        });
                        executor.execute();
                    }
                }
            }
            catch (e) {
                next(e);
            }
        });
    }
}
exports.DataRenderingSubMiddleware = DataRenderingSubMiddleware;
DataRenderingSubMiddleware.logger = logger_1.Logger.getLogger("hornet-js-core.middlewares.DataRenderingMiddleware");
const base_error_1 = require("hornet-js-utils/src/exception/base-error");
const business_error_1 = require("hornet-js-utils/src/exception/business-error");
const technical_error_1 = require("hornet-js-utils/src/exception/technical-error");
// ------------------------------------------------------------------------------------------------------------------- //
//                                      UnmanagedDataErrorMiddleware
// ------------------------------------------------------------------------------------------------------------------- //
class UnmanagedDataErrorMiddleware extends AbstractHornetMiddleware {
    constructor() {
        super((err, req, res, next) => {
            // route de type 'DATA' uniquement
            if (hornet_js_utils_1.Utils.getCls("hornet.routeType") === abstract_routes_1.RouteType.DATA) {
                sanitizeErrorThrownInDomain(err);
                if (!(err instanceof base_error_1.BaseError)) {
                    err = new technical_error_1.TechnicalError(err.code || "ERR_TECH_UNKNOWN", { errorMessage: err.message }, err);
                }
                if (err instanceof technical_error_1.TechnicalError) {
                    UnmanagedDataErrorMiddleware.logger.error("ERREUR technique [" + err.code + "] - reportId [" + err.reportId + "] : ", err);
                }
                else if (err instanceof business_error_1.BusinessError) {
                    UnmanagedDataErrorMiddleware.logger.error("ERREUR métier [" + err.code + "] - : ", err);
                }
                else {
                    UnmanagedDataErrorMiddleware.logger.error("ERREUR [" + err.code + "] - : ", err);
                }
                res.status(200);
                if (hornet_js_utils_1.Utils.config.getOrDefault("server.rethrow", false)) {
                    if (err.status && typeof err.status === "number") {
                        res.status(err.status);
                    }
                    else if (err.args && err.args.httpStatus && typeof err.args.httpStatus === "number") { // gestion avec les NodeApiError
                        res.status(err.args.httpStatus);
                    }
                    else if (err.httpStatus && typeof err.httpStatus === "number") { // gestion avec les HttpError
                        res.status(err.httpStatus);
                    }
                    else {
                        res.status(500);
                    }
                }
                res.json(service_api_results_1.NodeApiResultBuilder.buildError(err, req && req.originalUrl));
                timer_1.Timer.stopTimer(timer_1.Timer.TIMER_SERVER_HTTP);
                timer_1.Timer.stopTimer(timer_1.Timer.TIMER_REQUEST);
                timer_1.Timer.logFinally("global", req.originalUrl, req.method, res.statusCode && res.statusCode.toString());
                res.end();
            }
            else {
                UnmanagedDataErrorMiddleware.logger.error("ERREUR technique [" + err.code + "] - reportId [" + err.reportId + "] : ", err);
                if (err.status && typeof err.status === "number") {
                    res.status(err.status);
                }
                else if (err.args && err.args.httpStatus && typeof err.args.httpStatus === "number") { // gestion avec les NodeApiError
                    res.status(err.args.httpStatus);
                }
                else {
                    res.status(500);
                }
                timer_1.Timer.stopTimer(timer_1.Timer.TIMER_REQUEST);
                timer_1.Timer.logFinally("global", req.originalUrl, req.method, res.statusCode && res.statusCode.toString());
                res.end();
            }
        });
    }
}
exports.UnmanagedDataErrorMiddleware = UnmanagedDataErrorMiddleware;
UnmanagedDataErrorMiddleware.logger = logger_1.Logger.getLogger("hornet-js-core.middlewares.UnmanagedDataErrorMiddleware");
class AliveMiddleware extends AbstractHornetMiddleware {
    /**
     * Constructeur
     *
     * @param middlewareFunction
     * @param prefix
     * @param config server configuration
     */
    constructor(middlewareFunction, prefix, config) {
        super((req, res, next) => {
            res.status(204).end();
        }, prefix || "alive");
    }
    /**
     * Méthode appelée automatiquement lors de l'initialisation du serveur afin d'ajouter un middleware
     * @param app
     */
    insertMiddleware(app) {
        app.get(this.prefix || "alive", this.middlewareFunction);
    }
}
exports.AliveMiddleware = AliveMiddleware;
exports.DEFAULT_HORNET_MIDDLEWARES = [
    AliveMiddleware,
    HornetContextInitializerMiddleware,
    LoggerTIDMiddleware,
    SecurityMiddleware,
    WelcomePageRedirectMiddleware,
    WithoutSlashPageRedirectMiddleware,
    StaticNodeHttpHeaderMiddleware,
    StaticPathMiddleware,
    StaticPathErrorMiddleware,
    BodyParserJsonMiddleware,
    BodyParserUrlEncodedMiddleware,
    SessionMiddleware,
    InternationalizationMiddleware,
    ChangeI18nLocaleMiddleware,
    wakeUpNodeSessionMiddleware,
    LoggerUserMiddleware,
    SetExpandedLayoutMiddleware,
    IsExpandedLayoutMiddleware,
    ExpandedLayoutHandlerMiddleware,
    CsrfMiddleware,
    MulterMiddleware,
    RouterServerMiddleware,
    MenuConfigMiddleware,
    UserAccessSecurityMiddleware,
    DataRenderingMiddleware,
    UnmanagedDataErrorMiddleware,
];
exports.DEFAULT_HORNET_MODULE_MIDDLEWARES = [
    HornetContextInitializerSubMiddleware,
    RouterServerSubMiddleware,
    DataRenderingSubMiddleware,
];
exports.DEFAULT_HORNET_API_MIDDLEWARES = [
    AliveMiddleware,
    HornetContextInitializerMiddleware,
    LoggerTIDMiddleware,
    SecurityMiddleware,
    WithoutSlashPageRedirectMiddleware,
    BodyParserJsonMiddleware,
    BodyParserUrlEncodedMiddleware,
    LoggerUserMiddleware,
    MulterMiddleware,
    RouterServerMiddleware,
    UserAccessSecurityMiddleware,
    DataRenderingMiddleware,
    UnmanagedDataErrorMiddleware,
];
class HornetMiddlewareList {
    constructor(middlewares = exports.DEFAULT_HORNET_MIDDLEWARES) {
        this.list = [];
        middlewares.forEach((middleware) => {
            this.list.push(middleware);
        });
    }
    addBefore(newMiddleware, middleware) {
        const idx = this.list.indexOf(middleware);
        if (idx === -1) {
            throw new Error("Le middleware de base n'a pas été trouvé dans le tableau de middlewares " +
                ">> impossible d'insérer le nouveau middleware avant.");
        }
        this.list.splice(idx, 0, newMiddleware);
        return this;
    }
    addRouterBefore(router, middleware) {
        const idx = this.list.indexOf(middleware);
        if (idx === -1) {
            throw new Error("Le middleware de base n'a pas été trouvé dans le tableau de middlewares " +
                ">> impossible d'insérer le nouveau middleware avant.");
        }
        this.list.splice(idx, 0, router);
        return this;
    }
    addAfter(newMiddleware, middleware) {
        const idx = this.list.indexOf(middleware);
        if (idx === -1) {
            throw new Error("Le middleware de base n'a pas été trouvé dans le tableau de middlewares " +
                ">> impossible d'insérer le nouveau middleware après.");
        }
        this.list.splice(idx + 1, 0, newMiddleware);
        return this;
    }
    remove(middleware) {
        const idx = this.list.indexOf(middleware);
        if (idx === -1) {
            throw new Error("Le middleware n'a pas été trouvé dans le tableau de middlewares " +
                ">> suppression impossible.");
        }
        this.list.splice(idx, 1);
        return this;
    }
}
exports.HornetMiddlewareList = HornetMiddlewareList;
class HornetRouter {
    constructor(prefix, router) {
        this.prefix = prefix;
        this.router = router;
    }
}
exports.HornetRouter = HornetRouter;

//# sourceMappingURL=middlewares.js.map
