"use strict";
/**
 * Copyright ou © ou Copr. Ministère de l'Europe et des Affaires étrangères (2017)
 * <p/>
 * pole-architecture.dnum-psi@diplomatie.gouv.fr
 * <p/>
 * Ce logiciel est un programme informatique servant à faciliter la création
 * d'applications Web conformément aux référentiels généraux français : RGI, RGS et RGAA
 * <p/>
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * <p/>
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 * <p/>
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 * <p/>
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 * <p/>
 * <p/>
 * Copyright or © or Copr. Ministry for Europe and Foreign Affairs (2017)
 * <p/>
 * pole-architecture.dnum-psi@diplomatie.gouv.fr
 * <p/>
 * This software is a computer program whose purpose is to facilitate creation of
 * web application in accordance with french general repositories : RGI, RGS and RGAA.
 * <p/>
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 * <p/>
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 * <p/>
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 * <p/>
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractRoutes = exports.DataRouteInfos = exports.PageRouteInfos = exports.RouteInfos = exports.RouteActionService = exports.RouteAction = exports.DEFAULT_METHOD = exports.DEFAULT_AUTHORIZATION = exports.PRIVATE_ROUTE = exports.PUBLIC_ROUTE = exports.RouteType = void 0;
/**
 * hornet-js-core - Ensemble des composants qui forment le coeur de hornet-js
 *
 * @author MEAE - Ministère de l'Europe et des Affaires étrangères
 * @version v5.5.0
 * @link git+https://github.com/diplomatiegouvfr/hornet-js.git
 * @license CECILL-2.1
 */
const hornet_js_utils_1 = require("hornet-js-utils");
const logger_1 = require("hornet-js-logger/src/logger");
const technical_error_1 = require("hornet-js-utils/src/exception/technical-error");
const codes_error_1 = require("hornet-js-utils/src/exception/codes-error");
const media_type_1 = require("hornet-js-core/src/protocol/media-type");
const assign = require("lodash.assign");
const isString = require("lodash.isstring");
const isUndefined = require("lodash.isundefined");
const findIndex = require("lodash.findindex");
const logger = logger_1.Logger.getLogger("hornet-js-core.routes.abstract-routes");
/** Routes type */
exports.RouteType = {
    PAGE: "PAGE_ROUTE",
    DATA: "DATA_ROUTE",
};
exports.PUBLIC_ROUTE = [];
exports.PRIVATE_ROUTE = ["*"];
exports.DEFAULT_AUTHORIZATION = exports.PRIVATE_ROUTE;
exports.DEFAULT_METHOD = "get";
class RouteAction {
    constructor() {
        /** Attributs de la route déclenchant l'action */
        this.attributes = {};
        /** Utilisateur connecté */
        this.user = hornet_js_utils_1.Utils.getCls("hornet.user");
    }
    /**
     * Renvoie l'objet contenant les éléments nécessaires à la validation des données transmises à cette action.
     * Renvoie null par défaut : à surcharger éventuellement dans la classe action implémentée.
     * @returns {null} une instance de ActionValidation ou null
     */
    getDataValidator() {
        return null;
    }
    /**
     * Renvoie les données entrantes éventuelles, récupérées par défaut directement dans le corps de la requête.
     * A sucharger si nécessaire.
     * @return {any} un objet contenant les données transmises à cette action
     */
    getPayload() {
        return this.req.body;
    }
    /**
     * Renvoie le MediaType issu de l'entête de la requête.
     * @return {any} un objet contenant les données transmises à cette action
     */
    getMediaType() {
        return media_type_1.MediaTypes.fromMime(this.req.get("Accept"));
    }
}
exports.RouteAction = RouteAction;
class RouteActionService extends RouteAction {
    getService() {
        return this.service;
    }
}
exports.RouteActionService = RouteActionService;
class RouteInfos {
    constructor(type, attributes = {}, service) {
        this.attributes = {};
        this.type = type;
        assign(this.attributes, attributes);
        if (service) {
            this.service = service;
        }
    }
    getRouteType() {
        return this.type;
    }
    getAttributes() {
        return this.attributes;
    }
    getService() {
        return this.service;
    }
}
exports.RouteInfos = RouteInfos;
class PageRouteInfos extends RouteInfos {
    constructor(viewComponent, attributes, service) {
        super(exports.RouteType.PAGE, attributes, service);
        this.viewComponent = viewComponent;
    }
    getService() {
        return this.service;
    }
    getViewComponent() {
        return this.viewComponent;
    }
}
exports.PageRouteInfos = PageRouteInfos;
class DataRouteInfos extends RouteInfos {
    constructor(action, attributes, service) {
        super(exports.RouteType.DATA, attributes, service);
        this.action = action;
    }
    getAction() {
        return this.action;
    }
}
exports.DataRouteInfos = DataRouteInfos;
/** Routes Declaration */
class AbstractRoutes {
    constructor() {
        this.pageRoutes = {};
        this.dataRoutes = {};
        this.lazyRoutes = {};
        this.subRoutes = {};
    }
    resolveAuthorizationAndMethod(authorizationOrMethod, method) {
        let auth;
        let meth;
        if (isString(authorizationOrMethod)) {
            auth = exports.DEFAULT_AUTHORIZATION;
            meth = authorizationOrMethod;
        }
        else {
            if (isUndefined(authorizationOrMethod)) {
                auth = exports.DEFAULT_AUTHORIZATION;
            }
            else {
                auth = authorizationOrMethod;
            }
            if (isString(method)) {
                meth = method;
            }
            else {
                meth = exports.DEFAULT_METHOD;
            }
        }
        return { authorization: auth, method: meth };
    }
    addPageRoute(path, handler, authorization) {
        const args = this.resolveAuthorizationAndMethod(authorization, exports.DEFAULT_METHOD);
        if (!this.pageRoutes[path]) {
            this.pageRoutes[path] = {};
        }
        this.pageRoutes[path][args.method.toLowerCase()] = { authorization: args.authorization, handler };
    }
    addDataRoute(path, handler, authorizationOrMethod, method) {
        const args = this.resolveAuthorizationAndMethod(authorizationOrMethod, method);
        if (!this.dataRoutes[path]) {
            this.dataRoutes[path] = {};
        }
        this.dataRoutes[path][args.method.toLowerCase()] = { authorization: args.authorization, handler };
    }
    addLazyRoutes(path, subRoutesFile) {
        this.lazyRoutes[path] = subRoutesFile;
    }
    getPageRoutes() {
        return this.pageRoutes;
    }
    getDataRoutes() {
        return this.dataRoutes;
    }
    getLazyRoutes() {
        return this.lazyRoutes;
    }
    addSubRoute(path, subRoutesFile) {
        this.subRoutes[path] = subRoutesFile;
    }
    getSubRoutes() {
        return this.subRoutes;
    }
    /**
     * Permet de charger les routes depuis une liste de répertoires
     * @param paths
     * @returns route module
     */
    getDefaultRouteLoader(paths) {
        return (name) => {
            logger.trace("defaultRouteLoader(" + name + ")");
            const index = findIndex(paths, function (path) {
                try {
                    return require.main.require(path + name);
                }
                catch (e) {
                    throw new technical_error_1.TechnicalError("ERR_TECH_" + codes_error_1.CodesError.ROUTE_ERROR, { errorMessage: e.message }, e.stack);
                }
            });
            if (index === -1) {
                logger.error("Unknow route " + name + " in paths " + paths);
                throw new technical_error_1.TechnicalError("ERR_TECH_" + codes_error_1.CodesError.ROUTE_ERROR, { errorMessage: codes_error_1.CodesError.DEFAULT_ERROR_MSG });
            }
            return require.main.require(paths[index] + name);
        };
    }
}
exports.AbstractRoutes = AbstractRoutes;

//# sourceMappingURL=abstract-routes.js.map
