"use strict";
/**
 * Copyright ou © ou Copr. Ministère de l'Europe et des Affaires étrangères (2017)
 * <p/>
 * pole-architecture.dnum-psi@diplomatie.gouv.fr
 * <p/>
 * Ce logiciel est un programme informatique servant à faciliter la création
 * d'applications Web conformément aux référentiels généraux français : RGI, RGS et RGAA
 * <p/>
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * <p/>
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 * <p/>
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 * <p/>
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 * <p/>
 * <p/>
 * Copyright or © or Copr. Ministry for Europe and Foreign Affairs (2017)
 * <p/>
 * pole-architecture.dnum-psi@diplomatie.gouv.fr
 * <p/>
 * This software is a computer program whose purpose is to facilitate creation of
 * web application in accordance with french general repositories : RGI, RGS and RGAA.
 * <p/>
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 * <p/>
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 * <p/>
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 * <p/>
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.BackendApiError = exports.NodeApiError = exports.NodeApiResultBuilder = void 0;
/**
 * hornet-js-core - Ensemble des composants qui forment le coeur de hornet-js
 *
 * @author MEAE - Ministère de l'Europe et des Affaires étrangères
 * @version v5.5.0
 * @link git+https://github.com/diplomatiegouvfr/hornet-js.git
 * @license CECILL-2.1
 */
const base_error_1 = require("hornet-js-utils/src/exception/base-error");
const technical_error_1 = require("hornet-js-utils/src/exception/technical-error");
const business_error_1 = require("hornet-js-utils/src/exception/business-error");
const business_error_list_1 = require("hornet-js-utils/src/exception/business-error-list");
class NodeApiResultBuilder {
    static build(jsonData, url) {
        return {
            hasTechnicalError: false,
            hasBusinessError: false,
            status: 200,
            url: url || "???",
            data: jsonData,
            errors: [],
        };
    }
    static buildError(error, url) {
        return {
            hasTechnicalError: error instanceof technical_error_1.TechnicalError,
            hasBusinessError: error instanceof business_error_1.BusinessError,
            status: (error.args["httpStatus"] && parseInt(error.args["httpStatus"], 10))
                || (error["httpStatus"] && parseInt(error["httpStatus"], 10)) || 200,
            url: url || "???",
            data: null,
            errors: [error],
        };
    }
}
exports.NodeApiResultBuilder = NodeApiResultBuilder;
class NodeApiError {
    constructor(date, code, name, details, args, reportId, backend, httpStatus) {
        this.backend = false;
        this.date = date;
        this.code = code;
        this.name = name;
        this.details = details;
        this.args = args;
        this.reportId = reportId;
        this.backend = backend;
        this.httpStatus = httpStatus;
        this.nodeApiErrorList = [];
    }
    /**
     * Renvoie le résumé des détails éventuels de l'erreur d'origine.
     * @param apiError erreur serveur
     * @returns {string} la première ligne de détail
     */
    static parseDetails(apiError) {
        let details;
        if (apiError.err_cause && apiError.err_cause.message) {
            /* apiErrors[i].err_cause.message contient toute la pile d'erreur côté serveur :
             * on récupère uniquement la première ligne car il n'est pas judicieux
             * (notamment pour des raisons de sécurité) d'afficher toute la pile
             * d'erreur serveur dans le log client */
            const lineEndIndex = apiError.err_cause.message.indexOf("\n");
            if (process.env.NODE_ENV !== "production") {
                details = apiError.err_cause.message;
            }
            else {
                if (lineEndIndex > 0) {
                    details = apiError.err_cause.message.substring(0, lineEndIndex);
                }
                else {
                    details = apiError.err_cause.message;
                }
            }
        }
        return details;
    }
    static parseError(apiErrors, httpStatus) {
        if (!Array.isArray(apiErrors) && apiErrors.name === "BusinessErrorList") {
            apiErrors = apiErrors["errors"];
        }
        else if (Array.isArray(apiErrors) && apiErrors.length === 1 && apiErrors[0].name === "BusinessErrorList") {
            apiErrors = apiErrors[0]["errors"];
        }
        if (Array.isArray(apiErrors) && apiErrors.length > 1) {
            // cas d'une liste d'erreurs
            const global = new NodeApiError();
            for (let i = 0; i < apiErrors.length; i++) {
                const details = NodeApiError.parseDetails(apiErrors[i]);
                global.nodeApiErrorList.push(new NodeApiError(apiErrors[i].date, apiErrors[i].code, apiErrors[i].name, details, apiErrors[i].args, apiErrors[i].reportId, apiErrors[i].backend, httpStatus));
            }
            return global;
        }
        else {
            // cas d'une erreur simple
            let singleError;
            if (Array.isArray(apiErrors)) {
                singleError = apiErrors[0];
            }
            else {
                singleError = apiErrors;
            }
            const details = NodeApiError.parseDetails(singleError);
            return new NodeApiError(singleError.date, singleError.code, singleError.name, details, singleError.args, singleError.reportId, singleError.backend, httpStatus);
        }
    }
    toJsError() {
        if (this.nodeApiErrorList.length === 0) {
            let error;
            const args = {};
            // backend args
            for (const i in this.args) {
                args[i] = this.args[i];
            }
            // default args
            if (!args["reportId"] && this.reportId) {
                args["reportId"] = this.reportId;
            }
            if (!args["httpStatus"] && this.httpStatus) {
                args["httpStatus"] = this.httpStatus;
            }
            // Error type
            if (this.name === "BusinessError") {
                error = new business_error_1.BusinessError(this.code, args);
            }
            else {
                error = new technical_error_1.TechnicalError(this.code, args, this.details ? new base_error_1.BaseError("", this.details) : null);
                if (this.httpStatus && (this.httpStatus < 200 || this.httpStatus > 299)) {
                    error.httpStatus = this.httpStatus;
                }
            }
            error.backend = this.backend;
            return error;
        }
        else {
            const error = new business_error_list_1.BusinessErrorList();
            let backend = false;
            for (let i = 0; i < this.nodeApiErrorList.length; i++) {
                const apiError = this.nodeApiErrorList[i];
                let suberror;
                const args = {};
                // backend args
                for (const i in apiError.args) {
                    args[i] = apiError.args[i];
                }
                // default args
                if (!args["reportId"] && apiError.reportId) {
                    args["reportId"] = apiError.reportId;
                }
                if (!args["httpStatus"] && apiError.httpStatus) {
                    args["httpStatus"] = apiError.httpStatus;
                }
                // Error type
                if (apiError.name === "BusinessError") {
                    suberror = new business_error_1.BusinessError(apiError.code, args);
                }
                else {
                    suberror = new technical_error_1.TechnicalError(apiError.code, args, apiError.details ? new base_error_1.BaseError("", apiError.details) : null);
                }
                suberror.backend = backend = apiError.backend;
                error.addError(suberror);
            }
            error.backend = backend;
            return error;
        }
    }
}
exports.NodeApiError = NodeApiError;
class BackendApiError {
    constructor(date, code, name, type, details, args, reportId, httpStatus, originalAttrs) {
        this.date = date;
        this.code = code;
        this.name = name;
        this.type = type;
        this.details = details;
        this.args = args;
        this.reportId = reportId;
        this.httpStatus = httpStatus;
        this.backendApiErrorList = [];
        this.originalAttrs = Object.assign(originalAttrs || {}, {});
        delete (this.originalAttrs["stack"]);
        Object.keys(this).forEach((attrib) => {
            delete (this.originalAttrs[attrib]);
        });
    }
    /**
     * Crée l'instance de BackendApiError à partir de l'objet JSON représentant la ou les erreurs
     * @param apiErrors erreurs[s] représentant
     * @param httpStatus
     * @returns {BackendApiError}
     */
    static parseError(apiErrors, httpStatus) {
        if (Array.isArray(apiErrors) && apiErrors.length > 1) {
            // cas d'une liste d'erreurs
            const global = new BackendApiError();
            for (let i = 0; i < apiErrors.length; i++) {
                global.backendApiErrorList.push(new BackendApiError(apiErrors[i].date, apiErrors[i].code, apiErrors[i].name, apiErrors[i].type, apiErrors[i].details, apiErrors[i].args, apiErrors[i].reportId, httpStatus, apiErrors[i]));
            }
            return global;
        }
        else {
            // cas d'une erreur simple
            let singleError;
            if (Array.isArray(apiErrors)) {
                singleError = apiErrors[0];
            }
            else {
                singleError = apiErrors;
            }
            return new BackendApiError(singleError.date, singleError.code, singleError.name, singleError.type, singleError.details, singleError.args, singleError.reportId, httpStatus || 200, singleError);
        }
    }
    toJsError() {
        if (this.backendApiErrorList.length === 0) {
            let error;
            const args = Object.assign(this.originalAttrs || {}, {});
            // backend args
            for (let i = 0; i < this.args.length; i++) {
                args["$" + i] = this.args[i];
            }
            // default args
            if (this.reportId) {
                args["reportId"] = this.reportId;
            }
            if (this.httpStatus) {
                args["httpStatus"] = this.httpStatus;
            }
            if (this.name) {
                args["errorMessage"] = this.name;
            }
            else {
                args["errorMessage"] = "";
            }
            // Error type
            if (this.type === "TechnicalException") {
                error = new technical_error_1.TechnicalError(this.code, args, this.details ? new base_error_1.BaseError("", this.details) : null);
                if (this.httpStatus !== 200) {
                    error.httpStatus = this.httpStatus;
                }
                error.backend = true;
            }
            else if (this.type === "BusinessException") {
                error = new business_error_1.BusinessError(this.code, args);
                error.backend = true;
            }
            else {
                error = new base_error_1.BaseError(this.code, this.originalAttrs["message"] || undefined, args);
            }
            return error;
        }
        else {
            const error = new business_error_list_1.BusinessErrorList();
            for (let i = 0; i < this.backendApiErrorList.length; i++) {
                const apiError = this.backendApiErrorList[i];
                let suberror;
                const args = Object.assign(this.originalAttrs || {}, {});
                // backend args
                for (let i = 0; i < apiError.args.length; i++) {
                    args["$" + i] = apiError.args[i];
                }
                // default args
                if (apiError.reportId) {
                    args["reportId"] = apiError.reportId;
                }
                if (apiError.httpStatus) {
                    args["httpStatus"] = apiError.httpStatus;
                }
                if (apiError.name) {
                    args["errorMessage"] = apiError.name;
                }
                else {
                    args["errorMessage"] = "";
                }
                // Error type
                if (apiError.type === "TechnicalException") {
                    suberror = new technical_error_1.TechnicalError(apiError.code, args, apiError.details ? new base_error_1.BaseError("", apiError.details) : null);
                }
                else if (apiError.type === "BusinessException") {
                    suberror = new business_error_1.BusinessError(apiError.code, args);
                }
                suberror.backend = true;
                error.addError(suberror);
            }
            error.backend = true;
            return error;
        }
    }
}
exports.BackendApiError = BackendApiError;

//# sourceMappingURL=service-api-results.js.map
